%Mstep.m                                                                                             100664     502     454         5610  6651063237  10522  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                  
function model = Mstep(model, D, h, g)
%
% model = Mstep(model, D, h, g)
%
% M-step of EM algorithm.
% returns model structure, given the previous model, the data in D,
% and the results of the E-step in h & g.
% (c) Roderick Murray-Smith, Jan 1999, DTU, Denmark

[N, diminput]   = size(D.inputs);
[N, dimschedin] = size(D.schedin);
if model.lmtype == 1
   [localouts, variances] = locmodel(model, D);
end   
paramvar = zeros(N,model.Nm);
gates    = gate(model.gating, D.schedin);


if model.learnalg  < 3
 for m=1:model.Nm
   sumh = sum(h(:,m));	% normalisation term ~ sum of relevant points

   if model.lmtype == 1   % linear local models
      % calculate local model parameters

 
      if model.learnalg == 2
         % local learning
         Q     	= (g(:,m)./(model.models(m).stddev.^2))*ones(1, diminput+1);
      else
         % EM learning
         Q     	= (h(:,m)./(model.models(m).stddev.^2))*ones(1, diminput+1);
      end
      if model.learnalg == 0 % RBF
         Phi   	= ones(N,1);
         Q = Q(:,1);
         invdesign = pinv((Q)'*Phi);
         w 	= (invdesign*(Q)'*D.outputs);
         model.models(m).theta  = zeros(diminput,1);
         model.models(m).offset = w(1);
         % find error variance
         modelerr = D.outputs - Phi*w; %sum(modelerr.^2)/sumh;

         temp 	= invdesign*(Phi.*Q)';
         model.models(m).Sigmatheta = ((temp'.*((modelerr.^2)))'*temp')./sumh;
      else
         Phi   	= [ones(N,1) D.inputs];
         invdesign = pinv((Phi.*Q)'*Phi);
         w 	= (invdesign*(Phi.*Q)'*D.outputs);
         model.models(m).theta  = w(2:diminput+1)';
         model.models(m).offset = w(1);

         % find error variance
         modelerr = D.outputs - Phi*w; %sum(modelerr.^2)/sumh;

         temp 	= invdesign*(Phi.*Q)';
         model.models(m).Sigmatheta = ((temp'.*((modelerr.^2)*ones(1,diminput+1)))'*temp')./sumh;
      end



      % use the trace of the parameter variance times their Hessian to estimate effective
      % local degrees of freedom
      localDOF = diminput+1; %trace((Phi.*Q)'*Phi*model.models(m).invhessian);
 
      smallnumber = 0.0001; % for numerical reasons

      % take into account the variance of model parameters

      for i=1:N
         paramvar(i,m) = h(i,m)'.*(Phi(i,:)*(model.models(m).Sigmatheta)*Phi(i,:)'); 
      end

      model.models(m).stddev = smallnumber + sqrt(sum(h(:,m)'*(modelerr.^2 ...
       +  paramvar(:,m)))./sumh); %(sumh - localDOF));
     
   elseif model.lmtype == 0	% mixture model
      model.models(m).weighting = sumh/length(h);
   end % if
  end
end


% update model centre

for m=1:model.Nm
   sumh = sum(h(:,m));
   model.gating(m).centre = h(:,m)'*D.schedin./sumh;

   differ = D.schedin - ...
            ones(N,1)*model.gating(m).centre;
   weight_term = h(:,m)*ones(1, dimschedin);
   model.gating(m).Sigma = ((differ./sumh).*weight_term)'*differ + ...
                           0.0001*eye(size(model.gating(m).Sigma));
end

Estep.m                                                                                             100664     502     454         1246  6647404524  10516  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                 

function [h, gates, Likelihood] = Estep(model, D)
% function [h, gates, Likelihood] = Estep(model, D)
% 
% estimates the posterior probability of the missing values
% and returns them in h. 
% gates variable is the unconditional probability of each data point
% belonging to a given model
% Likelihood is loglikelihood of model
%
% (c) Roderick Murray-Smith, Jan 1999, DTU, Denmark

normalise =  ~(model.lmtype==0);
gates = gate(model.gating, D.schedin, normalise);

pY = proby(model, D);

h1 = gates.*pY;
h1 = h1 + eps*ones(size(h1));
h  = h1./(sum(h1')'*ones(1,model.Nm));

Likelihood = sum(log(sum(h1')));
%-sum(sum((h*(log(gates')+log(pY')))'))
%L2 = sum(log(sum((h.*pY)')))                                                                                                                                                                                                                                                                                                                                                          MoGdisplay.m                                                                                        100664     502     454         2056  6647412511  11501  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                     figure(1)   
   clf
   plot(1:i,Likelihood)
   title('Mixture model log-likelihood')

   figure(2)
   clf
   if dimsched == 1
      if diminput == 1
         clf
         subplot(2,1,1)
         Nbin = 30;
         [Nh,Xh] = hist(trainingdata.inputs,Nbin);
         bar(Xh,Nh/((Xh(2)-Xh(1))*N))
         hold on
         plotmodels(model, inputstest, trainingdata);
         hold off
         title('Mixture model, data histogram')
         subplot(2,1,2)
         plotgates(model, inputstest);
         title('Mixture components')
      end
     % hold on
     % plotgates(model, inputstest)
     % hold off
   elseif dimsched == 2
      plot(trainingdata.inputs(:,1),trainingdata.inputs(:,2),'.');
      hold on
      gates = plotgates(model, inputstest);
      title('Contour plot of component mixtures and training data');
      hold off
      if model.lmtype > 0 %diminput == 2
         if rem(i,3) == 0
           figure(2)
           fprintf('Plotting response')
           plotmodels(model, inputstest, trainingdata);
         end
      end
   end
   drawnow
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  README                                                                                              100664     502     454          340  6647404754  10117  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                  There are three files to try in this directory

%demomoe.m                                                                                       
%100664     502     454        12665  6651314660  11106  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                 
% demomoe
%
% This program uses EM to maximise the likelihood of a mixture of local models
% for fitting a regression-like data set.
%
% Questions & Tasks:
% try running from different initial conditions. 
% Are the results the same? Should they be?
% try the model with different numbers of components. Any interesting behaviour?
% Any observations on the behaviour of the likelihood history? 
% 
% In one dimensional problems, can you see the way the prior probabilities
% (probability of component i being the right one given ONLY x)
% indicated by the solid mixture component plots follow the posterior 
% probabilities (probability of component i being the right one given BOTH 
% x & y) indicated by dashed lines?
% For noisy data, do you understand why the dashed lines look noisy when the 
% data is noisy?
%
%
% (Note if you get an error complaining about likelihood decreasing it is my 
% problem - a numerical bug in the code somewhere. Sorry, just run it again!)
%
% How does the model cope with dataset 3 (the 10 dimensional data set used
% in other labs)? Why do you think this is?
% 
% try setting the variable model.learnalg = 2 (which brings in local learning 
% instead of standard EM optimisation. What effect does this have? 
% (The only difference in implementation is in the weighting function for the 
% local optimisation of local model prameters. In EM we seuse the the 
% posterior h(x), and in local learning we use the prior g(x), so the local 
% approach doesn't allow neighbouring models to 'cooperate' to better fit the
% target function.



clear;

% Initialise data set
dataset = 1;

if dataset == 1		% one dimensional data
   trainingdata.inputs = [-1:0.03:1]';
   trainingdata.schedin = trainingdata.inputs;
   N = length(trainingdata.inputs(:,1));
   trainingdata.outputs = sin(10*pi*( trainingdata.inputs(:,1)+ones(N,1)).*sin( trainingdata.inputs(:,1)/2).^3)+ 0.1*randn(N,1); 
    inputstest = [-1:0.05:1]';
elseif dataset == 2	% two dimensional data
   load trains1   % try also trains2
   trainingdata.inputs = trains1(:,1:2);
   trainingdata.schedin = trainingdata.inputs;
   trainingdata.outputs = trains1(:,3);
   inputstest = [-1.57:0.05:1.57]';
elseif dataset == 3	% ten dimensional data, first 3 of which are nonlinear
			% the next two linear to output, then last 5 no effect
                        % on output
   close all
   % you can experiment with number of inputs used 
   scheduse = [1:10];	% try reducing this to 1:3
   inpuse   = [1:10];	% try reducing this to 1:5
   load train200
   trainingdata.inputs = train200(:,inpuse);
   trainingdata.outputs = train200(:,11);
   trainingdata.schedin = trainingdata.inputs(:,scheduse);

   load test10000
   testdata.inputs = test10000(:,inpuse);
   testdata.schedin = testdata.inputs(:,scheduse);
   testdata.outputs = test10000(:,11);
   inputstest = [0:0.05:1]';
   fprintf('Note that mean error in this example is on test set, not on training data.\n');
end
[N, diminput]     = size(trainingdata.inputs);
[dummy, dimsched] = size(trainingdata.schedin);
[dummy, dimoutput] = size(trainingdata.outputs); 


% set up model
model.Nm     = 7;	% Number of components. Feel free to change this!
model.learnalg = 1;  	% 1 - EM, 2 - Local. Feel free to change this.
model.lmtype = 1;	% Select mixture of linear models
maxIter      = 100;	% Maximum number of EM iterations allowed
model.dimsched = dimsched;
model.models = struct('theta', cell(model.Nm,1), ...
                      'offset', 0, ...
		      'stddev', ones(1, dimoutput), ...
                      'Sigmatheta', eye(diminput+1,diminput+1));
for m = 1:model.Nm
    a = randperm(N); % pick Nm reandom points for centres
    model.gating(m).centre = trainingdata.schedin(a(m),:);
    model.models(m).theta = randn(dimoutput, diminput);
 end




% set the mixture means to randomly chosen data points.
model.gating = struct('centre', cell(model.Nm,1), ...
                      'Sigma', sqrt(0.01)*eye(dimsched, dimsched));
for m = 1:model.Nm
   model.models(m).weighting = 1./model.Nm;
   a = randperm(N); % pick Nm random points for centres
   model.gating(m).centre = trainingdata.schedin(a(m),:) + 0.01*randn(1,dimsched);
end


% start identification of model parameters
Likelihood  = [];
Meanerrlist = [];
h 	    = [];
finished    = 0;
i           = 1;
%inputstest  = [-1.5:0.05:1.5];

gates = gate(model.gating, trainingdata.schedin);
model = Mstep(model, trainingdata, gates, gates);


while i < maxIter & ~finished

   % E- & M-step. These two functions are the core of the identification
   % algorithm
   [h, gates, thisLike] = Estep(model, trainingdata);
   model 		= Mstep(model, trainingdata, h, gates);

   Likelihood = [Likelihood; thisLike];

   % rest of loop is purely graphic displays
   displaymoe;
   drawnow

   if i > 2
      if Likelihood(end) < Likelihood(end-1)
         fprintf('Likelihood decreased! Probably due to numerical problem in code. %f->%f\n',Likelihood(end-1),Likelihood(end))
         finished = 1;
      elseif  Likelihood(end) < Likelihood(end-1) + 0.001
         fprintf('Converged\n')
         finished = 1;
      end
   end
   i = i + 1;
end



% Display surface plots of the model & its individual components
if dimsched == 2
fprintf('Plotting response of individual basis functions in 3D\n')
x = [-1.5:0.02:1.5];
[X, Y] = meshgrid(x,x);
gates = plotgates(model, x);

figure(4)
clf
for i = 1:model.Nm
   surf(X,Y,reshape(gates(:,i),length(x), length(x)))
   hold on
end
hold off
shading interp
title('P.d.f. of individual gating functions')
rotate3d on
end

demomog.m                                                                                           100664     502     454        10703  6651315172  11076  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                 
% demomog	- Mixture of Gaussians demo.
%
% (c) Roderick Murray-Smith, Jan 1999, DTU, Denmark
%
% This program uses EM to maximise the likelihood of a mixture of Gaussians for
% modelling the probability density function of a dataset.
% Contour plots are 1 & 3 standard deviations away from the mean of the 
% component.
%
% Questions & Tasks:
% try running from different initial conditions. 
% Are the results the same? Should they be?
% try the model with different numbers of components. Any interesting behaviour?
% Use the hist() command to draw historgrams of your data. Change the number
% of bins used (e.g. N = hist(data, M) for M bins. Looking at the histogram, do
% you always have the same number of clusters, at different bin sizes?
% Any observations on the behaviour of the likelihood history?
% Trt running with dataset set first to 1 & then 2 to see both one and 
% two-dimensional examples.
%
% Try creating your own training data.
%
% to examine the parameters after training you can look at:
% model			data structure containing all parameters
% model.gating		data structure describing mixture components
% model.gating.Sigma	covariance matrix of components
% model.gating.centre	mean of components
% model.models.weighting weighting coefficient for each mixture component


clear;

% Initialise data set
dataset = 1; % can be 1 or 2 to choose one or two dimensional examples

if dataset == 1		% one dimensional data
   trainingdata.inputs = [ones(100,1)*[-0.5]+randn(100,1)*0.1;
                          randn(40,1)*0.1;
                          ones(80,1)*[0.5]+randn(80,1)*0.4];
elseif dataset == 2	% two dimensional data
   trainingdata.inputs = [ones(100,1)*[-0.5 0]+randn(100,2)*0.1;
                          randn(40,2)*0.1;
                          ones(80,1)*[0.5 0.7]+randn(80,2)*0.4];
end
trainingdata.outputs = [];
trainingdata.schedin = trainingdata.inputs;
[N, diminput]     = size(trainingdata.inputs);
[dummy, dimsched] = size(trainingdata.schedin);
dimoutput = 0; 

% set up model
model.Nm     = 3;	% Number of components
model.lmtype = 0;	% Select mixture of Gaussians model (dont change)
maxIter      = 100;	% Maximum number of EM iterations allowed
model.dimsched = dimsched;
model.gating   = struct('centre', cell(model.Nm,1), ...
                        'Sigma', sqrt(0.001)*eye(dimsched, dimsched));
model.learnalg = 1;  % 1 - EM

if model.Nm < 2
  fprintf('model.Nm must be greater than 1\n')
  break
end
% set the mixture means to randomly chosen data points.
for m = 1:model.Nm
   model.models(m).weighting = 1./model.Nm;
   a = randperm(N); % pick Nm random points for centres
   model.gating(m).centre = trainingdata.schedin(a(m),:)+ 0.01*randn(1,dimsched);
end


% start identification of model parameters
Likelihood  = [];
Meanerrlist = [];
h 	    = [];
finished    = 0;
i           = 1;
inputstest  = [-1.5:0.05:1.5]';

gates = gate(model.gating, trainingdata.schedin);
model = Mstep(model, trainingdata, gates, gates);


while i < maxIter & ~finished

   % E- & M-step. These two functions are the core of the identification
   % algorithm
   [h, gates, thisLike] = Estep(model, trainingdata);
   model 		= Mstep(model, trainingdata, h, gates);

   Likelihood = [Likelihood; thisLike];

   % rest of loop is purely graphic displays
   MoGdisplay;

   %eval(['print -f2 -depsc EMiter' num2str(i) '.eps'])
   if i > 2
      if Likelihood(end) < Likelihood(end-1)
         fprintf('Likelihood decreased! Numerical problems? %f->%f\n',Likelihood(end-1),Likelihood(end))
         finished = 1;
      elseif  Likelihood(end) < Likelihood(end-1) + 0.001
         fprintf('Converged OK\n')
         finished = 1;
      end
   end
   fprintf('Likelihood %f\n', thisLike);
   i = i + 1;
end




% Display surface plots of the model & its individual components
if dimsched == 2
   fprintf('Preparing 3D surface plots..\n')
   x = [-1.5:0.05:1.5];
   [X, Y] = meshgrid(x,x);
   gates = plotgates(model, x);

   figure(3)
   clf
   outputs = zeros(length(gates(:,1)),1);
   for i = 1:model.Nm
      surf(X,Y,reshape(gates(:,i),length(x), length(x)))
      hold on
      outputs = outputs + gates(:,i).*model.models(i).weighting;
   end
   hold off
   shading interp
   title('P.d.f. of individual components')
   rotate3d on

   figure(4)
   surf(X,Y,reshape(outputs,length(x), length(x)))
   shading interp
   title('Prob. density function of Mixture of Gaussians model')
   fprintf('You can now rotate the final p.d.f with your mouse to examine it. (Uses matlab command rotate3d\n')
   rotate3d on

end

displaymoe.m                                                                                        100664     502     454         3600  6651315113  11566  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                    figure(1)
   subplot(2,1,1)
   plot(1:i,Likelihood)
   title('Mixture model log-likelihood')


   if dimsched == 1
      if diminput == 1
         figure(2)
         clf
         subplot(2,1,2)
         hold on
         plot(trainingdata.inputs, h,'--')
         plotgates(model, inputstest);
         hold off
         title('Mixture components')
         subplot(2,1,1)       
         plot(trainingdata.inputs,trainingdata.outputs,'.');
         plotmodels(model, inputstest, trainingdata);
         title('Training data, local models and model output, with confidence intervals')
  
      end
     % hold on
     % plotgates(model, inputstest)
     % hold off
   elseif dimsched == 2
      figure(2)
      clf
      plot(trainingdata.inputs(:,1),trainingdata.inputs(:,2),'.');
      hold on
      gates = plotgates(model, inputstest);
      hold off
      if model.lmtype > 0  & diminput == 2
         if rem(i,1) == 0
           figure(3)
           clf
           fprintf('Plotting response\n')
           plotmodels(model, inputstest, trainingdata);
           title('Model mean response')
         end
      end
   end
   drawnow
if ~(model.lmtype == 0)
   %figure(5)
   %clf
   if ~exist('testdata')
      [modoutputs, modgates, modvariance] = meanLMN(model, trainingdata, trainingdata );
      meanerr = mean(((trainingdata.outputs - modoutputs).^2));
     % plot(1:N,modoutputs,':',1:N,trainingdata.outputs,'.')
   else
      [modoutputs, modgates, modvariance] = meanLMN(model, testdata, trainingdata );
      meanerr = mean(((testdata.outputs - modoutputs).^2));
     % plot(1:length(modoutputs),modoutputs,':',1:length(testdata.outputs),testdata.outputs,'.')
   end
   figure(1)
   Meanerrlist = [Meanerrlist; meanerr];
   fprintf('Iteration %d, mean err %f, mean var. %f, Likelihood %f\n',i, meanerr, mean(modvariance), thisLike);
   subplot(2,1,2)
   plot(1:i, Meanerrlist)
   title('Mean error')

end

gate.m
100664     502     454         1277  6647404524  10362  0                                                                                                    ustar   rod                             users    
function gates = gate(gating, inputs, normalise)

% returns the probability of each local model being correct for the given inputs

if nargin < 3
   normalise = 1;
end
[N,diminput] = size(inputs);
Nm           = length(gating);
gates 	     = zeros(N,Nm);
dist         = zeros(N,1);
gaussian = 1;

if gaussian
   for m = 1:Nm
      differ = inputs-ones(N,1)*gating(m).centre;
      Sigma = gating(m).Sigma;
      Sigmainv = inv(Sigma);
      for i=1:N
         dist(i) = differ(i,:)*Sigmainv*differ(i,:)';
      end
      gates(:,m) = exp(-dist/2)./(eps+sqrt(det(Sigma))*(2*pi)^(diminput/2));

   end %  for m
   if normalise
      gates = gates./(eps+sum(gates')'*ones(1,Nm));
   end
end % if gaussian

globMstep.m                                                                                         100664     502     454        11557  6647404524  11420  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                               
function model = globMstep(model, D, h, g)

[N, diminput]   = size(D.inputs);
[N, dimschedin] = size(D.schedin);
if model.lmtype == 1
   [localouts, variances] = locmodel(model, D);

end   
paramvar = zeros(N,model.Nm);
gates    = gate(model.gating, D.schedin);

if model.learnalg == 3
   % Global learning
   Phi = kron(ones(1,model.Nm),[ones(N,1) D.inputs]).*...
         kron(gates, ones(1,diminput+1));
   invPhi = pinv(Phi,1e-6);
   w 	= invPhi*D.outputs;
   Stheta = ((invPhi'.*(((D.outputs - Phi*w).^2)*ones(1,model.Nm*(diminput+1))))'*invPhi')./N;
   for m=1:model.Nm
      model.models(m).theta  = w(2*m:2*m+diminput-1)';
      model.models(m).offset = w(2*m-1);
      model.models(m).Sigmatheta = diag(Stheta(2*m:2*m+diminput-1,(2*m:2*m+diminput-1)))';
   end
elseif model.learnalg == 4
   % Tikhonov regularised learning
   % Global learning
   Phi = kron(ones(1,model.Nm),[ones(N,1) D.inputs]).*...
         kron(gates, ones(1,diminput+1));
   [u,s,v] = svd(Phi);
   [w,rho,eta] = tikhonov(u,diag(s),v,D.outputs,model.lambda);
   figure(4)
   l_curve(u,diag(s),D.outputs,'tikh');
   figure(5)
   eta = picard(u,diag(s),D.outputs,1);


   F_tikh = fil_fac(diag(s),model.lambda);
   invTikh = v*diag(F_tikh./diag(s))*u(:,1:length(F_tikh))';
   Stheta = ((invTikh'.*(((D.outputs - Phi*w).^2)*ones(1,model.Nm*(diminput+1))))'*invTikh')./N;
   for m=1:model.Nm
      model.models(m).theta  = w(2*m:2*m+diminput-1)';
      model.models(m).offset = w(2*m-1);
      model.models(m).Sigmatheta = diag(Stheta(2*m:2*m+diminput-1,(2*m:2*m+diminput-1)))';
   end


elseif model.learnalg  < 3
   
 for m=1:model.Nm
   sumh = sum(h(:,m));	% normalisation term ~ sum of relevant points

   if model.lmtype == 1   % linear local models
      % calculate local model parameters

      Phi   	= [ones(N,1) D.inputs];
      if model.learnalg == 2
         % local learning
         Q     	= (g(:,m)./(model.models(m).stddev.^2))*ones(1, diminput+1);
      else
         % EM learning
         Q     	= (h(:,m)./(model.models(m).stddev.^2))*ones(1, diminput+1);
      end
      invdesign = pinv((Phi.*Q)'*Phi);
      w 	= (invdesign*(Phi.*Q)'*D.outputs);
      model.models(m).theta  = w(2:diminput+1)';
      model.models(m).offset = w(1);

      % find error variance
      modelerr = D.outputs - Phi*w;
sum(modelerr.^2)/sumh;

      temp 	= invdesign*(Phi.*Q)';
      model.models(m).Sigmatheta = ((temp'.*((modelerr.^2)*ones(1,diminput+1)))'*temp')./sumh;

      % use the trace of the parameter variance times their Hessian to estimate effective
      % local degrees of freedom
      localDOF = diminput+1; %trace((Phi.*Q)'*Phi*model.models(m).invhessian);
 
      smallnumber = 0.0001; % for numerical reasons

      % take into account the variance of model parameters

    %  for i=1:N
    %     paramvar(i,m) = h(i,m)'.*(Phi(i,:)*(model.models(m).Sigmatheta)*Phi(i,:)');
    % end

      model.models(m).stddev = smallnumber  + sqrt(sum(h(:,m)'*(modelerr.^2 ...
       +  paramvar(:,m)))./sumh); %(sumh - localDOF));
     
   elseif model.lmtype == 0	% mixture model

      model.models(m).weighting = sumh/length(h);

     % model.models(m).switchweight = model.models(m).switchweight + ...
     %     0.001*sum(-log(h1(:,m))*((model.models(m).switchweight./model.sumweights)*model.Pmgrad + model.Pm)); 
     % model.models(m).switchweight = max(model.models(m).switchweight,0);
     % model.models(m).switchweight = min(model.models(m).switchweight,1);


   elseif model.lmtype == 2 	% Gaussian Process
      % learn hyperparameters
   %   for m=1:model.Nm
          meanx = sum(D.inputs.*h(:,m)*ones(1,diminput))./sum(h(:,m));
          meany = sum(D.outputs.*h(:,m))./sum(h(:,m));
          x = D.inputs - ones(N,1)*meanx;
          y = D.outputs - ones(N,1)*meany;
          pos = find(h(:,m)>0.05);

          % randomly find 50 from available inputs
          endpos = length(pos);%min(length(pos),50);
          randpos = randperm(length(pos));

          [mu, sigmasq, L, Sigma, Sigmainv] = gaussinf(x(pos(randpos(1:endpos),:)), ...
               x(pos(randpos(1:endpos),:)), y(pos(randpos(1:endpos))), model.models(m).gp);

         % model.models(m).gp.weighting = (h(pos(randpos(1:endpos)),m)./sigmasq)';%./exp(model.models(m).gp.noise);
         model.models(m).gp.weighting = [];
         
          weightvector = struct2vector(model.models(m).gp);
          %[d,dy,dh] = check('gaussmin',weightvector, x(randpos(1:endpos),:), y(randpos(1:endpos)), length(model.models(m).gp.noise));
          
          [weightvector, Likelihood, iters, numfunc] = ...
             cgmin2(weightvector,'gaussmin', 10, x(randpos(1:endpos),:), y(randpos(1:endpos)), length(model.models(m).gp.noise),(h(pos(randpos(1:endpos)),m))');
          
          model.models(m).gp = vector2struct(weightvector, diminput, length(model.models(m).gp.noise));
          model.models(m).gp.meanx = meanx;
          model.models(m).gp.meany = meany;
      %end % m
   end % if
  end
end
  
  
  initdat.m   
  100664     502     454        12450  6647404524  11111  0   
  ustar   rod                             users               
  test = 1;

if test == 1
inputstest = [-1:0.1:1]';
inputs = [0.7*randn(250,1)+ones(60,1)*[0.5 0.5] + 0.3*randn(60,2)];
schedin = inputs;
outputs = sin(2*(inputs(:,1).^3))+0.4*inputs(:,2);

[N, diminput] 	= size(inputs);
[N2, dimsched]	= size(schedin);
dimcontrol 	= 0;
dimoutput 	= 1;
model.lmtype  = 1;

elseif test == 3
  load henrikdata

  %inputst = in_l';
  %outputst = out';
  %schedint = in_b';

  inputs  = chop_dat(in_l,5,1)';
  outputs = chop_dat(out,5,0)';
  deadtime = 1;

  inputs  = [inputs(2:end-2) inputs(1:end-3) outputs(1+deadtime:end-1-deadtime) outputs(deadtime:end-2-deadtime)];
  outputs = outputs(2+deadtime:end-2+deadtime);
  schedin = chop_dat(in_b,5,0)';
  schedin = schedin(2:end-2);
  clear in_b in_l out;

  [N, diminput]	= size(inputs);
  [N2, dimsched]= size(schedin);
  dimcontrol 	= 0;
  dimoutput 	= 1;
  inputstest    = [min(schedin):0.02:max(schedin)]';
  model.lmtype  = 1;

elseif test == 4
   inputstest = [-1:0.05:1]';
   inputs = [-1:0.03:1]';
   schedin = inputs;
   [N, diminput] 	= size(inputs);
   [N2, dimsched]	= size(schedin);

   %outputs = inputs+exp(sin(2.6*inputs));
 
   outputs = sin(10*pi*(inputs(:,1)+ones(N,1)).*sin(inputs(:,1)/2).^3); %sin((1.5*inputs(:,1)).^3);
  outputs = outputs + 0.4*randn(size(outputs));

   dimcontrol 	= 0;
   dimoutput 	= 1;
   model.lmtype = 1;
elseif test == 5

   % wind speed 1 wind speed 2 wind direction ambient air temp. 
   % power
   tpredict = 3*60/5;
   load winddata

   inputs  = [inputs(1:end-tpredict-20000,:) outputs(1:end-tpredict-20000)];
   outputs = outputs(tpredict+1:end-20000,:);
   inputs(:,3) = cos(2*pi*inputs(:,3)/360);
   avwindspeed = sum(inputs(:,1:2)')'/2;
   inputs = [avwindspeed avwindspeed.^2 inputs(:,3:end)];
   [N, diminput] = size(inputs);

   inputs  = inputs - ones(N,1)*mean(inputs);
   inputs  = inputs./(ones(N,1)*std(inputs));
   %outputs = outputs - ones(N,1)*mean(outputs);
   outputs = outputs./(ones(N,1)*max(outputs));

   schedin = inputs; %[inputs(:,1) inputs(:,4)];
   inputstest = [-2:0.05:2]';
   [N2, dimsched]	= size(schedin);
   dimcontrol 	= 0;
   dimoutput 	= 1;
   model.lmtype = 1;

elseif test == 6
   % coffee data

   %inputstest = [-1:0.05:1]';
   %inputs = [randn(50,3)*0.2*[1 0.3 0; 0.3 0.5 0; 0 0 1] - [ zeros(50,1) ones(50,1)*2 zeros(50,1)];
   %          randn(100,3)*0.1 + [ones(100,1)*1 zeros(100,2)]];
   schedin = inputs;
   [N, diminput] 	= size(inputs);
   [N2, dimsched]	= size(schedin);
   inputstest = [];
   dimoutput = 0;
   outputs = [];
   model.lmtype = 0;		% Select mixture model
elseif test == 7
   inputs = [-0.1*ones(100,1)+randn(100,1)*0.2; randn(10,1)*0.9; 0.1*ones(10,1)+randn(10,1)*0.9];
   schedin = inputs;
   [N, diminput]  = size(inputs);
   [N2, dimsched] = size(schedin);
   inputstest = [randn(100,1)*0.2; randn(10,1)*0.9];
   dimoutput = 0;
   outputs = [];
   model.lmtype = 0;		% Select mixture model
elseif test == 8
    % set weights to useful initial values
    diminput = 3;
    dimoutput = 1;
    load /home/rod/mat/dynsim/pentest;

    %outputstest = [y(2:end)'];% x(2:end)'];
    inputstest = [-pi:0.05:pi]';%[x(1:end-1)' y(1:end-1)' u(1:end-1)'];

    load /home/rod/mat/dynsim/penprbs;
yorig = y';
xorig = x';
    thetatrain = yorig(2:end); 
    omegatrain = xorig(2:end);
    x = [xorig(1:end-1) yorig(1:end-1) u(1:end-1)']; 
    N = length(x(:,1));%300;
    i = randperm(length(thetatrain));
    inputs = x(i(1:N),:);
    outputs = [thetatrain(i(1:N),:)];% omegatrain(i(1:N),:)];
    schedin = inputs(:,1:2);
    [N,diminput] = size(inputs);
    [N2, dimsched] = size(schedin);
    model.lmtype = 1;
if 0
    load /home/rod/mat/dynsim/penprbs;
yorig = y';
xorig = x';
    thetatrain = yorig(2:end); 
    omegatrain = xorig(2:end);
    x = [xorig(1:end-1) yorig(1:end-1) u(1:end-1)'];
  %  N = 1000;
   % i = randperm(length(thetatrain));
   % x = x(i(1:N),:);
    %thetatrain = thetatrain(i(1:N),:);
    %omegatrain = omegatrain(i(1:N),:);
end
elseif test == 9
   load ../torengdata4;
   inputstest = [-1:0.05:1]';
   inputs = [u(1:end-1)' y(1:end-1)'/max(y)];
   schedin = inputs;
   [N, diminput] 	= size(inputs);
   [N2, dimsched]	= size(schedin);
   outputs =diff(y');%./max(y);

   dimcontrol 	= 0;
   dimoutput 	= 1;
   model.lmtype = 1;
elseif test == 10
   load rambolldata.txt;
   dat = rambolldata;


   N = length(dat(:,1));
   hours24 = floor(24*1*60/5);
   predictoffset = hours24;

   dates = datenum(ones(N,1)*1900+dat(:,3),dat(:,1),dat(:,2),dat(:,4),dat(:,5),0.00001*ones(N,1));
   cumfjv = zeros(N,1);
   for i = 1:N-hours24
      cumfjv(i) = sum(dat(i:i+hours24,8));
   end
   days = datestr(dates,9); % return a vector of strings for days of the week
   daysnum = ones(size(days));
   % saturday/sunday
   I = find(days=='S');
   daysnum(I) = 0*ones(size(I));
   % Friday 
   I = find(days=='F');
   daysnum(I) = 0.3*ones(size(I));
   % Monday
   I = find(days=='M');
   daysnum(I) = 0.6*ones(size(I));



   inputs = [daysnum(1+predictoffset:end,1) dat(1+predictoffset:end,([9 14:17])) dat(1:end-predictoffset,([9 14:17]))];
   inputs = normalise(inputs, [min(inputs)' max(inputs)']);
   inputstest = inputs;
   schedin = inputs;
   [N, diminput] 	= size(inputs);
   [N2, dimsched]	= size(schedin);
   outputs = cumfjv(1:end-predictoffset);
   outputs = normalise(outputs, [min(outputs)' max(outputs)']);
   dimcontrol 	= 0;
   dimoutput 	= 1;
   model.lmtype = 1;
   clear rambolldata dat;

end



locmodel.m                                                                                          100664     502     454         1131  6651063342  11217  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                               
function [localouts, variances] = locmodel(model, dataset)


[N,w] = size(dataset.inputs);

localouts = zeros(N, model.Nm);
%gates     = gate(model.gating, dataset.schedin);


for m=1:model.Nm
   localouts(:,m) = dataset.inputs*model.models(m).theta' + ...
                    ones(N,1)*model.models(m).offset;

   dev      = [dataset.inputs ones(N,1)];

   % take into account the variance of model parameters
   paramvar = zeros(N,1);
   for i=1:N
      paramvar(i) = dev(i,:)*(model.models(m).Sigmatheta)*dev(i,:)';
   end

   variances(:,m) = (model.models(m).stddev.^2)*ones(N,1) + paramvar;
end
                                                                                                                                                                                                                                                                                                                                                                                                                                       meanLMN.m                                                                                           100664     502     454          757  6651063326  10707  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                  
function [outputs, gates, variance] = meanLMN(model, dataset, trainingdata)

% calculates the mean output for the given inputs and model.

global GPweighting;

[N,diminput] = size(dataset.inputs);

gates        = gate(model.gating, dataset.schedin);


if model.lmtype == 1
   [localouts, variances]  = locmodel(model, dataset);
end

outputs      = sum((gates.*localouts)')';

%variance     = sum((gates.*variances)')';
variance     = sum((gates.*(variances + localouts.^2))')' - (outputs.^2);

normalise.m                                                                           
100664     502     454         1124  6647404524  11422  0     
ustar   rod                             users                                                                                                                                                                                                              
function normed_input = normalise(raw_input, new_limits)

range = (new_limits(:,2)-new_limits(:,1));
pos_zero = find(range ==0);
no_zero = find(range ~=0);

if ~isempty(pos_zero)
   raw_input(:,pos_zero) = zeros(length(raw_input(:,1)), length(pos_zero));
end
if ~isempty(no_zero)
   normed_input(:,no_zero) = (raw_input(:,no_zero)-ones(length(raw_input(:,no_zero)),1)*new_limits(no_zero,1)')./(ones(length(raw_input(:,no_zero)),1)*range(no_zero)');
end
%normed_input   = 2.*normed_input - ones(normed_input);

if ~isempty(pos_zero)
  fprintf('There were inputs with zero content')
  pos_zero
end

plotgates.m                                                                                         100664     502     454         2027  6647404524  11436  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                 
function gates = plotgates(model, x)


normalise =  ~(model.lmtype==0);
if model.dimsched == 1
   gates = gate(model.gating, x, normalise);
   plot(x, gates)

elseif model.dimsched == 2
   [X, Y] = meshgrid(x,x);
   inputs = [X(:) Y(:)];
   gates = gate(model.gating, inputs, normalise);
   for m=1:model.Nm
      contour(X, Y, reshape(gates(:,m),length(x),length(x)), [exp(-1) exp(-3)]./(eps+sqrt(det(model.gating(m).Sigma))*(2*pi)^(model.dimsched/2)))
      plot(model.gating(m).centre(:,1), model.gating(m).centre(:,2), 'o')
      hold on
   end
   hold off
   axis('square')   
elseif model.dimsched == 3
   N = 20;
   [X,Y,Z] = sphere(N-1);
   A = zeros(N.^2,3);
   for m=1:model.Nm
      for i=1:N.^2
         A(i,:) = [X(i) Y(i) Z(i)]*2*chol(model.gating(m).Sigma);
      end
      A = A + ones(size(X(:)))*model.gating(m).centre;
      X1 = reshape(A(:,1),N,N);
      Y1 = reshape(A(:,2),N,N);
      Z1 = reshape(A(:,3),N,N);
      surfl(X1, Y1, Z1);
      %material shiny
      %shading interp
      hold on
   end
   hold off
end
drawnow                                                                                                                                                                                                                                                                  

plotmodels.m      
100664     502     454         3117  6647404524  11617  0  
ustar   rod                             users       
function modelout = plotmodels(model, inputs, trainingdata)

if model.dimsched == 1


   % draw local models
   hold on
   for m = 1:model.Nm
      minx = model.gating(m).centre - 2*sqrt(model.gating(m).Sigma);
      maxx = model.gating(m).centre + 2*sqrt(model.gating(m).Sigma);

      if model.lmtype == 1
         meanlm = [1 1]*(model.models(m).offset)+model.models(m).theta.*[minx maxx];
         % plot mean of local model between +/-2 std. devs
         plot([minx maxx]', meanlm);
         % plot confidence band of local model (at 1 std. deviation).
        % plot([minx maxx]', 2*[1 1]*(model.models(m).stddev)+meanlm,':');
        % plot([minx maxx]', -2*[1 1]*(model.models(m).stddev)+meanlm,':');
     end
   end

   % plot model response

   dataset.inputs  = inputs;
   dataset.schedin = inputs;
   if model.lmtype ~= 0
      [modelout, g, variance]  = meanLMN(model, dataset,trainingdata);

      plot(inputs, modelout,'r-.', inputs, modelout-2*sqrt(variance), 'c--', inputs, modelout+2*sqrt(variance), 'c--');
   elseif model.lmtype == 0
      gates = gate(model.gating, inputs, 0);
      modelout = zeros(length(gates(:,1)),1);
      for i = 1:model.Nm
         modelout = modelout + gates(:,i).*model.models(i).weighting;
      end
      plot(inputs,modelout);
   end

elseif model.dimsched == 2
   [X, Y] = meshgrid(inputs, inputs);
   dataset.inputs  = [X(:) Y(:)];
   dataset.schedin = dataset.inputs;
   modelout  = meanLMN(model, dataset, trainingdata);
   
   mesh(X,Y,reshape(modelout,length(inputs), length(inputs)));
end
hold off

ax = axis;
ax(1:2) = [min(inputs) max(inputs)];
axis(ax)                                                                                                                                                                                                                                                                                                                                                                                                                                                 proby.m                                                                                             100664     502     454         1043  6647404524  10564  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                 
function pY = proby(model, dataset)

[N,w] = size(dataset.inputs);

if model.lmtype == 1  		% linear local models
   [localouts, variances]  = locmodel(model, dataset);

   for m=1:model.Nm
      pY(:,m) = exp(-0.5*((dataset.outputs - localouts(:,m)).^2)./ ...
                          variances(:,m))./ ...
                (sqrt(2*pi)*sqrt(variances(:,m)));
      pY(:,m) = pY(:,m) + eps*ones(size(pY(:,m)));
   end

elseif model.lmtype == 0  	% mixture model
   for m=1:model.Nm
      pY(:,m) = ones(N,1)*model.models(m).weighting;
   end
end

                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             