%**************************************************************
% Modeling a Three-Input Nonlinear Function
% Output = (1+x^0.5+y^-1+z^-1.5)
% Foi tambem usada por Takagi, Sugeno, Kondo
% 216 pontos de treinamento uniformemente distribuido no Intervalo [1,6]x[1,6]x[1,6]
% 216 pontos de validao randomicamente distribuido [1,6]x[1,6]x[1,6]
% 125 pontos de teste randomicamente distribuido [1.5,5.5]x[1.5,5.5]x[1.5,5.5]
% Artigo
% 1) Adaptive-Network-Based Fuzzy Inference System
% Jyh-Shing Roger Jang
% 
% 2)T. Takagi and M. Sugeno. Derivation of fuzzy control rules from human 
% operators control actions. Proc. of the IFAC Symp. on Fuzzy Information, 
% Knowledge Representation and Decision Analysis, pages 5560, July 1983.
% 
% 3)M. Sugeno and G. T. Kang. Structure identification of fuzzy model. Fuzzy 
% Sets and Systems, 28:1533, 1988.
%
% 4) T. Kondo. Revised GMDH algorithm estimating degree of the complete polynomial. 
% Tran. of the Society of Instrument and Control Engineers, 22(9):928934, 1986. 
% (Japanese).
%*************************************************************

function dados_jang

nrun= 10; % Numero de simulaes realizadas

Ntr=216;
Nval=216;
Nts=125;

for i=1:nrun,
   
   % ====== collect training data
   point_n = 6;
   x = linspace(1, 6, point_n);
   y = linspace(1, 6, point_n);
   z = linspace(1, 6, point_n);
   
   count = 1;
   for l = x,
      for j = y;
         for k = z;
            out = (1+l^0.5+j^(-1)+k^(-1.5))^2;
            Xtr(count, :) = [l j k ];
            Ytr(count,1)=out;
            count = count + 1; 
         end
      end
   end
   
   % ====== collect selection data
   Nval=216;
   Xval=1+5*rand(Nval,3);
   x=Xval(:,1);
   y=Xval(:,2);
   z=Xval(:,3);
   Yval=(1+x.^(0.5)+y.^(-1)+z.^(-1.5)).^2;
   
   
   
   % ====== collect test data
   Nts=125;
   Xts=1.5+4*rand(Nval,3);
   x=Xts(:,1);
   y=Xts(:,2);
   z=Xts(:,3);
   Yts=(1+x.^(0.5)+y.^(-1)+z.^(-1.5)).^2;
   
  if i==1
      save dados1_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==2
      save dados2_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==3
      save dados3_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==4
      save dados4_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==5
      save dados5_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==6
      save dados6_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==7
      save dados7_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==8
      save dados8_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==9
      save dados9_jang Xtr Ytr Xts Yts Xval Yval
   elseif i==10
      save dados10_jang Xtr Ytr Xts Yts Xval Yval
   end

end

[N,m]=size(Xtr);
N
if m==2
   x1=reshape(Xtr(:,1),sqrt(N),sqrt(N));   
   x2=reshape(Xtr(:,2),sqrt(N),sqrt(N));
   Y=reshape(Ytr,sqrt(N),sqrt(N));
   surf(x1,x2,Y)
end


   
