%**************************************************************
% Utiliza gradiente conjugado escalonado hibrido
% Hessiana calculado pelo metodo exato
% n = [numero de unidades ocultas , numero de saidas];
% X = entrada (numero de entradas x numero de amostras)
% S = saida desejada (numero de amostras X pelo numero de saidas)
% MISTURA DE ESPECIALISTA - REDE GATING
% Autor: Clodoaldo Aparecido de Moraes Lima
%*************************************************************
function [A,B]=treina_gating(x,y,A,B,w,h)

% Calcula o numero de entradas e saidas
[N,ne]=size(x');
[N,ns]=size(y');

% Monta o vetor n
n=[h,ns];

%Inicializa o vetor de erros
nerro=[];

%Define o erro minimo
erromin=1e-5;

% Define o numero de epocas maximo
nepocamax=20;

%Inicializa a epoca inicial
nepocas=0;

% Constantes do algoritmo
lambatual=0;       % pode variar de 0 -- 10e-6
lambant=0;         % lambda barra
sucesso=1;         % 1 - verdaddeiro, 0 - falso

% Inicializa o contador
cont=1;

% Calcula o numero de parmetros
npar=h*(ne+1)+ns*(h+1);

% Calcula o gradiente
%[Ew,dEw] = processa_gating(X,S,A,B,n,ne,N);
dEw = processa_gating(x',y',A,B,w',n,ne,N);

post=computePost(A,B,x);

Ew=-computeLogLik(post,y,w);

% Armazena o valor do erro
EQM=Ew;
nerro=[nerro,EQM];


pat=-dEw;
rat=-dEw;
rant=-dEw;

%while EQM>erromin & nepocas<nepocamax
   	
while  nepocas<nepocamax
    
   if sucesso ==1
      %Calculando a infromao de Segunda Ordem
      vetdJdAv=pat(1:h*(ne+1));
      vetdJdBv=pat(1+h*(ne+1):end);
      Av=reshape(vetdJdAv,ne+1,h)';
      Bv=reshape(vetdJdBv,h+1,ns)';
      s=hprocessa_gating(x',y',A,B,Av,Bv,w',n,ne,N);
      pHp=pat'*s;
      normapat=pat'*pat;
      delta=pHp;
   end
   
   delta=delta+(lambatual-lambant)*normapat;
   
   if delta <=0
      lambant=lambatual;
      lambatual=2*(lambant-delta/normapat);
      delta=-delta+lambant*normapat;
    end
      
   mi=pat'*rat;
      
   alfa=mi/delta;
      
   Aaux=A+alfa*Av;
   Baux=B+alfa*Bv;
   
   %[Ewaux,dEwaux] = processa_gating(X,S,Aaux,Baux,n,ne,N);
   
   dEwaux = processa_gating(x',y',Aaux,Baux,w',n,ne,N);
   
   post2 = computePost(Aaux,Baux,x);
   
   Ewaux=-computeLogLik(post2,y,w);

   Ewp=Ew+alfa*pat'*dEw+(1/2)*(alfa*pHp*alfa);
   
   deltacomp=(Ew-Ewaux)/(Ew-Ewp);   %Ewv --> calculado corretamente
   
     
   %pause
   if deltacomp >= 0      
      
       disp(sprintf('  Conjugado Gating iter=%d lli=%g stepsize=%g norm (g) = %g',nepocas,-Ew,alfa,norm(dEw)));
       
       nepocas=nepocas+1;
       
      lambant=0;
      
      sucesso=1;
      
      A=Aaux;
      
      B=Baux;
      
      post=post2;
         
      dEw=dEwaux;
      
      Ew=Ewaux;
            
      rant=rat;
      
      rat=-dEw;
      
      EQM=Ew;
      nerro=[nerro,EQM];
      
      if cont == npar
         beta=0;
         pat=rat;
         cont=1;
      else      
         betaPR=(rat'*rat-rat'*rant)/(rant'*rant);
         betaFR=rat'*rat/(rant'*rant);
         
         if betaPR>0 & betaPR<betaFR
            pat=rat+betaPR*pat;
            cont=cont+1;
            beta=betaPR;
            
            if (pat'*rat)<0
               beta=0;
               pat=rat;
               cont=1;
               disp('Alfa ser negativo')
            end
            
            %elseif betaPR>betaFR
         elseif betaPR>betaFR
            pat=rat+betaFR*pat;
            cont=cont+1;
            beta=betaFR;
            
            if (pat'*rat)<0
               beta=0;
               pat=rat;
               cont=1;
               disp('Alfa ser negativo')
            end
            
         else
            beta=0;
            pat=rat;
            cont=1;
            disp('Reinicializando')   
         end
                 
      end
      
      if deltacomp >= 0.75
         lambatual=(1/4)*lambatual;
      end
      
   else
      
      sucesso=0;
         
      if deltacomp < 0.25
         lambant=lambatual;
         lambatual=lambant+delta*(1-deltacomp)/normapat;
      end
      
   end
   
   yr = perceptrons_gating(x',A,B);
   % Plota o grafico da saida
   figure(1)
   clf
   plot(y(1,:)')
   hold on
   plot(yr(:,1),'r--')
   grid
	drawnow      
end


%figure(2)
%clf
%plot(S,'b')
%grid
%hold on
%plot(Sr,'r--')
%title('Sada Desejada(azul) x Sada da Rede Neural(ver)')
%xlabel('Dados de Treinamento')
%drawnow

%figure(3)
%clf
%plot(0:length(nerro)-1,nerro)
%grid
%title('Erro Quadrtico Mdio (EQM)')
%xlabel('Nmero de pocas')
%ylabel('EQM')
%drawnow
%pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% class posteriors
function post = computePost(A,B,x)
  [d,n] = size(x);
  %  [d,k] = size(beta);
  k=size(B,1);  
  post = zeros(k,n);
  bx = zeros(k,n);
  %  for j = 1:k, 
  %    bx(j,:) = beta(:,j)'*x; 
  %  end
  bx=perceptrons(x',A,B);
  
  bx=bx'; % Realiza transposicao

  for j = 1:k, 
    post(j,:) = 1 ./ sum(exp(bx - repmat(bx(j,:),k,1)),1);
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% log likelihood
function lli = computeLogLik(post,y,w)
  [k,n] = size(post);
  lli = 0;
  for j = 1:k,
    lli = lli + sum(w.*y(j,:).*log(post(j,:)+eps));
  end
  if isnan(lli), 
    error('lli is nan'); 
  end


