%************************************************************************
% function model = treina_model(X,S,model,h,ne,ns,N)
% Para mistura de especialista
% Metodo de apresentacao dos padroes em lote
% Mtodo Gradiente Conjugado hibrido
% Calcula o produto H*d
%************************************************************************

function rdJdx = hprocessa(X,Yd,model,grad,ne,ns,N)


%Calcula a saida para todos especialistas
for i=1:model.Nm,
   Y(:,:,i)=perceptrons_expert(X,model.models(i).A,model.models(i).B);
end

h = calch(X,Yd,model,ns);

%Calcula a saida para a rede gating
%g=perceptrons_gating(X,model.gating.A,model.gating.B);

%Copia o numero de neuronios
hp=model.hp;
hg=model.hg;

%Calcula o numero de padres da rede especialista
npar_expert= (ne+1)*hp+(hp+1)*ns;

%Calcula o numero de padres da rede gating
npar_gating= (ne+1)*hg+(hg+1)*model.Nm;

%Calcula o numero total de parametros
npar_total=npar_expert*model.Nm+npar_gating;

%Matriz para armazenar as derivadas
for i=1:model.Nm,
     dJdr(:,i)=zeros(npar_expert,1); 
     rdJdr(:,i)=zeros(npar_expert,1); 

end
dJdz=zeros(npar_gating,1);
rdJdz=zeros(npar_gating,1);

for k=1:N,
   
   %----------------------------------------------------------------------------
   % Calcula a saida para cada especialista
	%----------------------------------------------------------------------------
   for i=1:model.Nm,     
     %Copia a matriz de pesos para as variaves w1 w2
      A=model.models(i).A;
      B=model.models(i).B;
   	%Calcula a saida para camada escondida   
      x1=[X(k,:)';1];
   	y1=tanh(A*x1);
      u=[y1;1];
      %Calcula a saida da  rede
      y2=B*u;
      %Calcula a o erro e as derivadas
      yd=Yd(k,:)';
      erro=yd-y2;
      %Calcula a variancia de cada especialista
      var=model.models(i).stddev.^2;
      %Calcula a derivada em relao a saida Y
      dJdY(:,i)=(h(k,i)/var)*erro;
      
      dYdA=[];
      dYdB=[];
      rdYdA=[];
	   rdYdB=[];

      %Calcula a derivada dos parametros em relao aos pesos da rede
      for j=1:hp,
         for l=1:ns,
            dYdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-u(j,1)*u(j,1))*x1;
         end
      end
      
      for l=1:ns,
         dYdB(1+(hp+1)*(l-1):(hp+1)*l,l)=u;
      end
      
      %Armazena as derivada
      dYdr(:,:,i)=[dYdA;dYdB];
     
      dJdr(:,i)=dJdr(:,i)+dYdr(:,:,i)*dJdY(:,i);
      
      grad_exp=grad(1+(i-1)*npar_expert:npar_expert*i,1);
      
      vetdJdA=grad_exp(1:hp*(ne+1),1);
      
      vetdJdB=grad_exp(1+hp*(ne+1):end,1);
      
      Av=reshape(vetdJdA,ne+1,hp)';
      
      Bv=reshape(vetdJdB,hp+1,ns)';

      rx1=zeros(ne+1,1);
      
      u=u(1:end-1);
      
      ru=(A*rx1+Av*x1).*(1-u.*u);
      
      rf=-2*u.*ru;
      
      ry(:,i)=Bv*[u;1]+B*[ru;0];
      
      for j=1:ns,
         rdYdB(1+(j-1)*(hp+1):j*(hp+1),j)=[ru;0];
      end
      

		for j=1:hp,
   		for l=1:ns,
            rdYdA(1+(j-1)*(ne+1):j*(ne+1),l)=rf(j,1)*x1*B(l,j)+(1-u(j,1)*u(j,1))*rx1*B(l,j)+(1-u(j)*u(j))*x1*Bv(l,j);
         end
      end
      
      rdYdr(:,:,i)=[rdYdA;rdYdB];
      
      
     end
   
     
   %-------------------------------------------------------------------------------------
   % Calcula a saida da rede gating
   %-------------------------------------------------------------------------------------
   %Copia a matriz de pesos para as variaves w1 w2
   A=model.gating.A;
   B=model.gating.B;
 	%Calcula a saida para camada escondida   
   x1=[X(k,:)';1];
  	y1=tanh(A*x1);
   u=[y1;1];
   %Calcula a saida da  rede
   s=B*u;
   %Normaliza
   g=exp(s)/sum(exp(s));
   %Calcula a o erro e as derivadas
   yd=h(k,:)';
   erro=yd-g;
   %Calcula deriva dJdY
   dJds=erro;
   
   dYdA=[];
   dYdB=[];
   rdYdA=[];
   rdYdB=[];

   for j=1:hg,
      for l=1:model.Nm,
         dSdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-u(j,1)*u(j,1))*x1;
      end
   end
      
   for l=1:model.Nm,
      dSdB(1+(hg+1)*(l-1):(hg+1)*l,l)=u;
   end
      
   dsdz=[dSdA;dSdB];
   
   dJdz=dJdz+dsdz*dJds; 
   
   grad_gat=grad(1+npar_expert*model.Nm:end);
   
   vetdJdA=grad_gat(1:hg*(ne+1));
   
   vetdJdB=grad_gat(1+hg*(ne+1):end);
   
   Av=reshape(vetdJdA,ne+1,hg)';
   
   Bv=reshape(vetdJdB,hg+1,model.Nm)';
 
   rx1=zeros(ne+1,1);
   
   u=u(1:end-1);
   
   ru=(A*rx1+Av*x1).*(1-y1.*y1);
      
   rf=-2*y1.*ru;
      
   rs=Bv*[u;1]+B*[ru;0];
   
   for j=1:model.Nm,
      rg(j,1)=g(j,1)*rs(j,1)-g(j,1)*(g'*rs);  
   end
      
   for j=1:model.Nm,
       rdSdB(1+(j-1)*(hg+1):j*(hg+1),j)=[ru;0];
    end
   

	for j=1:hg,
   	for l=1:model.Nm,
         rdSdA(1+(j-1)*(ne+1):j*(ne+1),l)=rf(j,1)*x1*B(l,j)+(1-u(j,1)*u(j,1))*rx1*B(l,j)+(1-u(j,1)*u(j,1))*x1*Bv(l,j);
      end
   end
   
   
   rdsdz=[rdSdA;rdSdB];
   
   
   %------------------------------------------------------------
   % Calculo R{h}
   %------------------------------------------------------------
    
    c=0;
    for j=1:model.Nm,
       var=model.models(j).stddev.^2;
       yd=Yd(k,:)';
       y2=Y(k,:,j)';
       erro=yd-y2;
       c=c+h(k,j)*(erro/var)'*ry(:,j);
    end
    
    for j=1:model.Nm,
      var=model.models(j).stddev.^2;
      yd=Yd(k,:)';
      y2=Y(k,:,j)';
      erro=yd-y2;
      rh1(j,1)=h(k,j)*(erro'/var)*ry(:,j)-h(k,j)*c;
      rh2(j,1)=h(k,j)*rg(j,1)/g(j,1)-h(k,j)*(h(k,:)*(rg./g));
      rh(j,1)=rh1(j,1)+rh2(j,1);
    end
    
 	
    rdJds=rh-rg;
    
    rdJdz=rdJdz+rdsdz*dJds+dsdz*rdJds;
    
    %--------------------------------------------------------------
    % Calculo de R{dJdr} para as redes especialista
    %--------------------------------------------------------------
    for j=1:model.Nm,
       var=model.models(j).stddev.^2;
       yd=Yd(k,:)';
       y2=Y(k,:,j)';
       erro=yd-y2;
       rdJdY=rh(j,1)*erro/var-h(k,j)*ry(:,j)/var;       
       rdJdr(:,j)= rdJdr(:,j)+rdYdr(:,:,j)*dJdY(:,j)+dYdr(:,:,j)*rdJdY;
    end
    
end

rdJdx=[];
for j=1:model.Nm,
   rdJdx=[rdJdx;rdJdr(:,j)];
end

rdJdx=-[rdJdx;rdJdz];

