%************************************************************************
% function model = treina_model(X,S,model,h,ne,ns,N)
% Para mistura de especialista
% Metodo de apresentacao dos padroes em lote
% Mtodo Gradiente Conjugado hibrido
% Calcula o produto H*d
%************************************************************************

function [Likelihood,dJdx] = processa(X,Yd,model,ne,ns,N)


%Calcula a saida para todos especialistas
for i=1:model.Nm,
   Y(:,:,i)=perceptrons_expert(X,model.models(i).A,model.models(i).B);
end

%Calcula a saida para a rede gating
%g=perceptrons_gating(X,model.gating.A,model.gating.B);
h = calch(X,Yd,model,ns);

%Copia o numero de neuronios
hp=model.hp;
hg=model.hg;

%Calcula o numero de padres da rede especialista
npar_expert= (ne+1)*hp+(hp+1)*ns;

%Calcula o numero de padres da rede gating
npar_gating= (ne+1)*hg+(hg+1)*model.Nm;

%Calcula o numero total de parametros
npar_total=npar_expert*model.Nm+npar_gating;

for i=1:model.Nm,
   dJdr(:,i)=zeros(npar_expert,1);
end
dJdz=zeros(npar_gating,1);


for k=1:N,
   
   %----------------------------------------------------------------------------
   % Calcula a saida para cada especialista
	%----------------------------------------------------------------------------
   for i=1:model.Nm,     
     %Copia a matriz de pesos para as variaves w1 w2
      A=model.models(i).A;
      B=model.models(i).B;
   	%Calcula a saida para camada escondida   
      x1=[X(k,:)';1];
   	y1=tanh(A*x1);
      u=[y1;1];
      %Calcula a saida da  rede
      y2=B*u;
      %Calcula a o erro e as derivadas
      yd=Yd(k,:)';
      erro=yd-y2;
      %Calcula a variancia de cada especialista
      var=model.models(i).stddev.^2;
      %Calcula a derivada em relao a saida Y
      dJdY(:,i)=(h(k,i)/var)*erro;
      
      dYdA=[];
      dYdB=[];
      %Calcula a derivada dos parametros em relao aos pesos da rede
      for j=1:hp,
         for l=1:ns,
            dYdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-y1(j,1)*y1(j,1))*x1;
         end
      end
      
      for l=1:ns,
         dYdB(1+(hp+1)*(l-1):(hp+1)*l,l)=u;
      end
      
      %Armazena as derivada
      dYdr(:,:,i)=[dYdA;dYdB];
     
      dJdr(:,i)=dJdr(:,i)+dYdr(:,:,i)*dJdY(:,i);
         
      
   end
   %-------------------------------------------------------------------------------------
   % Calcula a saida da rede gating
   %-------------------------------------------------------------------------------------
   %Copia a matriz de pesos para as variaves w1 w2
   A=model.gating.A;
   B=model.gating.B;
 	%Calcula a saida para camada escondida   
   x1=[X(k,:)';1];
  	y1=tanh(A*x1);
   u=[y1;1];
   %Calcula a saida da  rede
   s=B*u;
   %Normaliza
   g=exp(s)/sum(exp(s));
   %Calcula a o erro e as derivadas
   yd=h(k,:)';
   erro=yd-g;
   %Calcula deriva dJdY
   dJds=erro;
   
   dSdA=[];
   dSdB=[];
   for j=1:hg,
      for l=1:model.Nm,
         dSdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-y1(j,1)*y1(j,1))*x1;
      end
   end
      
   for l=1:model.Nm,
      dSdB(1+(hg+1)*(l-1):(hg+1)*l,l)=u;
   end
      
   dsdz=[dSdA;dSdB];
   
   dJdz=dJdz+dsdz*dJds;     
 
end

dJdx=[];
for i=1:model.Nm,
   dJdx=[dJdx;dJdr(:,i)];
end

dJdx=-[dJdx;dJdz];

Likelihood = -fobjetiva(X,Yd,model,N,ns);

