%************************************************************************
% function model = treina_model(X,S,model,h,ne,ns,N)
% Para mistura de especialista
% Metodo de apresentacao dos padroes em lote
% Mtodo gradiente
%************************************************************************

function model = treina_model(X,Yd,model,h,ne,ns,N)

alfa=1;

% Calcula o likelihood anterior
Likelihood_ant = fobjetiva(X,Yd,model,N,ns);

%Calcula a saida para todos especialistas
for i=1:model.Nm,
   Y(:,:,i)=perceptrons_expert(X,model.models(i).A,model.models(i).B);
end

%Calcula a saida para a rede gating
g=perceptrons_gating(X,model.gating.A,model.gating.B);

%Copia o numero de neuronios
hp=model.hp;
hg=model.hg;

%Calcula o numero de padres da rede especialista
npar_expert= (ne+1)*hp+(hp+1)*ns;

%Calcula o numero de padres da rede gating
npar_gating= (ne+1)*hg+(hg+1)*model.Nm;

%Calcula o numero total de parametros
npar_total=npar_expert*model.Nm+npar_gating;

%Matriz para armazenar as derivadas
for i=1:model.Nm,
     dYdr(:,:,i)=zeros(npar_expert,ns);      
     dJdr(:,i)=zeros(npar_expert,1); 
end

dJdz=zeros(npar_gating,1);

for k=1:N,
   
   %----------------------------------------------------------------------------
   % Calcula a saida para cada especialista
	%----------------------------------------------------------------------------
   for i=1:model.Nm,     
     %Copia a matriz de pesos para as variaves w1 w2
      A=model.models(i).A;
      B=model.models(i).B;
   	%Calcula a saida para camada escondida   
      x=[X(k,:)';1];
   	y1=tanh(A*x);
      u=[y1;1];
      %Calcula a saida da  rede
      y2=B*u;
      %Calcula a o erro e as derivadas
      yd=Yd(k,:)';
      erro=yd-y2;
      %Calcula deriva dJdY
      var=model.models(i).stddev.^2;
      dJdY=(h(k,i)/var)*erro;
      
      for j=1:hp,
         for l=1:ns,
            dYdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-u(j,1)*u(j,1))*x;
         end
      end
      
      for l=1:ns,
         dYdB(1+(hp+1)*(l-1):(hp+1)*l,l)=u;
      end
      
      dYdr(:,:,i)=[dYdA;dYdB];
      
      
      dJdr(:,i)=dJdr(:,i)+dYdr(:,:,i)*dJdY; 
   end
   
      
   %-------------------------------------------------------------------------------------
   % Calcula a saida da rede gating
   %-------------------------------------------------------------------------------------
   %Copia a matriz de pesos para as variaves w1 w2
   A=model.gating.A;
   B=model.gating.B;
 	%Calcula a saida para camada escondida   
   x=[X(k,:)';1];
  	y1=tanh(A*x);
   u=[y1;1];
   %Calcula a saida da  rede
   y2=B*u;
   %Normaliza
   g=exp(y2)/sum(exp(y2));
   %Calcula a o erro e as derivadas
   yd=h(k,:)';
   erro=yd-g;
   %Calcula deriva dJdY
   dJds=erro;
   
   for j=1:hg,
      for l=1:model.Nm,
         dSdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-u(j,1)*u(j,1))*x;
      end
   end
      
   for l=1:model.Nm,
      dSdB(1+(hg+1)*(l-1):(hg+1)*l,l)=u;
   end
      
   dsdz=[dSdA;dSdB];
   dJdz=dJdz+dsdz*dJds; 
    
end

%--------------------------------------------------------------------------------
%convert as derivadas em vetor
%--------------------------------------------------------------------------------
vetdJdx=[];
% Rede especialista
for i=1:model.Nm,
   vetdJdx=[vetdJdx;dJdr(:,i)];
end

% Rede Gating
vetdJdx=[vetdJdx;dJdz];

%--------------------------------------------------------------------------------
%convert os pesos em vetor
%--------------------------------------------------------------------------------
vetx=convert_vetor(model,ns,ne);

d=vetdJdx;

vet_peso=vetx+d;

%--------------------------------------------------------------------------------
%convert os pesos em matriz
%--------------------------------------------------------------------------------

model=convert_matriz(model,vet_peso,ne,ns);

%--------------------------------------------------------------------------------
%verifica o erro
%--------------------------------------------------------------------------------

Likelihood_atual = fobjetiva(X,Yd,model,N,ns);

while Likelihood_atual<Likelihood_ant
   alfa=alfa/2;
	vet_peso=vetx+alfa*d;
   model=convert_matriz(model,vet_peso,ne,ns);
   Likelihood_atual = fobjetiva(X,Yd,model,N,ns);
end


Likelihood_ant;

Likelihood_atual

alfa;

smallnumber = 0.0001; % for numerical reasons
Y=[];
for i=1:model.Nm,
   
   sumh = sum(h(:,i));	% normalisation term ~ sum of relevant points

   Y=perceptrons_expert(X,model.models(i).A,model.models(i).B);

	modelerr=Yd-Y;

  % model.models(i).stddev = smallnumber + sqrt(sum(h(:,i)'*(modelerr.^2 ))./sumh); %(sumh - localDOF));
end




  