%demomog.m                                                                                          
%100664     502     454        10703  6651315172  11076  0                                                                                                    ustar   rod                             users                                                                                                                                                                                                                  % demomog	- Mixture of Gaussians demo.
%
% (c) Roderick Murray-Smith, Jan 1999, DTU, Denmark
%
% This program uses EM to maximise the likelihood of a mixture of Gaussians for
% modelling the probability density function of a dataset.
% Contour plots are 1 & 3 standard deviations away from the mean of the 
% component.
%
% Questions & Tasks:
% try running from different initial conditions. 
% Are the results the same? Should they be?
% try the model with different numbers of components. Any interesting behaviour?
% Use the hist() command to draw historgrams of your data. Change the number
% of bins used (e.g. N = hist(data, M) for M bins. Looking at the histogram, do
% you always have the same number of clusters, at different bin sizes?
% Any observations on the behaviour of the likelihood history?
% Trt running with dataset set first to 1 & then 2 to see both one and 
% two-dimensional examples.
%
% Try creating your own training data.
%
% to examine the parameters after training you can look at:
% model			data structure containing all parameters
% model.gating		data structure describing mixture components
% model.gating.Sigma	covariance matrix of components
% model.gating.centre	mean of components
% model.models.weighting weighting coefficient for each mixture component


clear;

% Initialise data set
dataset = 2; % can be 1 or 2 to choose one or two dimensional examples

if dataset == 1		% one dimensional data
   trainingdata.inputs = [ones(100,1)*[-0.5]+randn(100,1)*0.1;
                          randn(40,1)*0.1;
                          ones(80,1)*[0.5]+randn(80,1)*0.4];
elseif dataset == 2	% two dimensional data
   trainingdata.inputs = [ones(100,1)*[-0.5 0]+randn(100,2)*0.1;
                          randn(40,2)*0.1;
                          ones(80,1)*[0.5 0.7]+randn(80,2)*0.4];
end
trainingdata.outputs = [];
trainingdata.schedin = trainingdata.inputs;
[N, diminput]     = size(trainingdata.inputs);
[dummy, dimsched] = size(trainingdata.schedin);
dimoutput = 0; 

% set up model
model.Nm     = 3;	% Number of components
model.lmtype = 0;	% Select mixture of Gaussians model (dont change)
maxIter      = 100;	% Maximum number of EM iterations allowed
model.dimsched = dimsched;
model.gating   = struct('centre', cell(model.Nm,1), ...
                        'Sigma', sqrt(0.001)*eye(dimsched, dimsched));
model.learnalg = 1;  % 1 - EM

if model.Nm < 2
  fprintf('model.Nm must be greater than 1\n')
  break
end
% set the mixture means to randomly chosen data points.
for m = 1:model.Nm
   model.models(m).weighting = 1./model.Nm;
   a = randperm(N); % pick Nm random points for centres
   model.gating(m).centre = trainingdata.schedin(a(m),:)+ 0.01*randn(1,dimsched);
end


% start identification of model parameters
Likelihood  = [];
Meanerrlist = [];
h 	    = [];
finished    = 0;
i           = 1;
inputstest  = [-1.5:0.05:1.5]';

gates = gate(model.gating, trainingdata.schedin);
model = Mstep(model, trainingdata, gates, gates);


while i < maxIter & ~finished

   % E- & M-step. These two functions are the core of the identification
   % algorithm
   [h, gates, thisLike] = Estep(model, trainingdata);
   model 		= Mstep(model, trainingdata, h, gates);

   Likelihood = [Likelihood; thisLike];

   % rest of loop is purely graphic displays
   MoGdisplay;

   %eval(['print -f2 -depsc EMiter' num2str(i) '.eps'])
   if i > 2
      if Likelihood(end) < Likelihood(end-1)
         fprintf('Likelihood decreased! Numerical problems? %f->%f\n',Likelihood(end-1),Likelihood(end))
         finished = 1;
      elseif  Likelihood(end) < Likelihood(end-1) + 0.001
         fprintf('Converged OK\n')
         finished = 1;
      end
   end
   fprintf('Likelihood %f\n', thisLike);
   i = i + 1;
end




% Display surface plots of the model & its individual components
if dimsched == 2
   fprintf('Preparing 3D surface plots..\n')
   x = [-1.5:0.05:1.5];
   [X, Y] = meshgrid(x,x);
   gates = plotgates(model, x);

   figure(3)
   clf
   outputs = zeros(length(gates(:,1)),1);
   for i = 1:model.Nm
      surf(X,Y,reshape(gates(:,i),length(x), length(x)))
      hold on
      outputs = outputs + gates(:,i).*model.models(i).weighting;
   end
   hold off
   shading interp
   title('P.d.f. of individual components')
   rotate3d on

   figure(4)
   surf(X,Y,reshape(outputs,length(x), length(x)))
   shading interp
   title('Prob. density function of Mixture of Gaussians model')
   fprintf('You can now rotate the final p.d.f with your mouse to examine it. (Uses matlab command rotate3d\n')
   rotate3d on

end


  


