%************************************************************************
% function model = processa(X,S,model,ne,ns,N)
% Para rede MLP estatica
% Fornece o erro quadratico medio na saida e o vetor gradiente 
% Metodo de apresentacao dos padroes em lote
% n: [numero de unidades ocultas  numero de saidas]
% m: numero de entradas (reais)
%************************************************************************

function model = processa(X,S,model,ne,ns,N)


%Calcula a saida para todos especialistas
for i=1:model.Nm,
   y(:,:,i)=perceptrons_expert(X,model.models(i).A,model.models(i).B);
end

%Calcula a saida para a rede gating
g=perceptrons_gating(X,model.gating.A,model.gating.B);

%Copia o numero de neuronios
hp=model.hp;
hg=model.hg;

%Calcula o numero de padres da rede especialista
npar_expert= (ne+1)*hp+(hp+1)*ns;

%Calcula o numero de padres da rede gating
npar_gating= (ne+1)*hg+(hg+1)*model.Nm;

%Calcula o numero total de parametros
npar_total=npar_expert*model.Nm+npar_gating;

%Incializa a matriz da ddJ/ddx
ddJddx=zeros(npar_total,npar_total);

for i=1:model.Nm
   dydr(:,:,i)=zeros(npar_expert,1);
end
dydz=zeros(npar_gating,1);


for k=1:N,
   for i=1:m,
      %Copia a matriz de pesos para as variaves w1 w2
      w1=model.models(i).A;
      w2=model.models(i).B;
   	%Calcula a saida para camada escondida   
      x1=[X(j,:)';1];
   	y1=tanh(w1*x1);
      x2=[y1;1];
      %Calcula a saida da  rede
      y2=w2*x2;
      %Calcula a o erro e as derivadas
	   erro=S(k,:)-y2;
	   erro2=erro;
   	dw2(:,:,i)= erro2*x2';
	   erro1=(w2(:,1:hp)'*erro2).*(1-y1.*y1);
   	dw1(:,:,i)=erro1*x1';
      dydr(:,:,i)=dydr(:,:,i)+[reshape(dw1',hp*(ns+1),1);reshape(dw2',ns*(hp+1),1)];
   end
   
   for i=1:m,
      for j=1:m,
         if i=~j
            ddJdydy=(S(k,:)-y(k,:,i))'*h(k,i)*h(k,j)*(S(k,:)-y(k,:,j));
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(j-1):npar_expert*j)=...
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(j-1):npar_expert*j)+...
            dydr(:,:,i)'*ddJdydy*dydr(:,:,j);
         else
            ddJddy=-h(k,i)*eye(ns)+(S(k,:)-y(k,:,i))'*(h(k,i)-h(k,i)^2)*(S(k,:)-y(k,:,i));
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(j-1):npar_expert*j)=...
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(j-1):npar_expert*j)+...
            dydr(:,:,i)'*ddJddy*dydr(:,:,i);
         end
      end
   end
   
   %Copia a matriz de pesos para as variaves w1 w2
   w1=model.gating.A;
   w2=model.gating.B;

 	x1=[X(i,:)';1];
   y1=tanh(w1*x1);
   x2=[y1;1];
   y2=w2*x2;
   g=exp(y2)/sum(exp(y2));
	erro=h(i,:)-g;
	erro2=erro;
   dw2=erro2*x2';
	erro1=(w2(:,1:hg)'*erro2).*(1-y1.*y1);
   dw1=erro1*x1';
   dydz=dydz+[reshape(dw1',hg*(ne+1),1);reshape(dw2',ns*(hg+1),1)];
         
   for i=1:m
      for j=1:m
         if i~=j 
            ddJddx(npar*m+i,npar*m+j)=ddJddx(npar*m+i,npar*m+j)+h(k,i)*h(k,j)+g(i,1)*g(j,1);
         else
            ddJddx(npar*m+i,npar*m+j)=ddJddx(npar*m+i,npar*m+j)+(h(k,i)-h(k,i)^2)-(g(i,1)-g(i,1)^2);
         end
      end
   end
end

%Calcula o valor da funo objetiva antes de atulizar o modelo
Likelihood_ant=fobjetiva(X,S,model);

%Coloca todas as derivadas num unico vetor
dydx=[dydr;dydz];

%Calcula a variao dos pesos
delta= inv(ddJddx-model.mi*eye(npar_total))*dydx;

% Transforma os pesos em matriz e depois atualiza
for i=1:model.Nm
   %Extrai os pesos correspondentes
   delta_A=delta(1+npar_expert*(i-1):hp*(ne+1)+npar_expert*(i-1));
   delta_B=delta(1+hp*(ne+1)+npar_expert*(i-1):npar_expert*i);
   %Tranforma em matriz
   mat_delta_A=reshape(delta_A',hp,ne+1);
   mat_delta_B=reshape(delta_B',ns,hp+1);
   %Atualiza os pesos
   model.models(i).A=model.models(i).A-mat_delta_A;
   model.models(i).B=model.models(i).B-mat_delta_B;
end

%Extrai os pesos da rede gating
delta_A=delta(1+npar_expert*model.Nm:hg*(ne+1)+npar_expert*model.Nm);
delta_B=delta(1+hg*(ne+1)+npar_expert*model.Nm:end);
%Transforma em matriz
mat_delta_A=reshape(delta_A',hg,ne+1);
mat_delta_B=reshape(delta_B',ns,hg+1);
%Atualiza os pesos
model.gating.A=model.gating.A-mat_delta_A;
model.gating.B=model.gating.A-mat_delta_B;

%Calcula o valor da funo objetiva depois de atualizar o modelo   
Likelihood_atual=fobjetiva(X,S,model);

if Likelihood_atual<Likelihood_atual % Sucesso
   model.mi=model.mi+5; 
else                   					 % Fracasso
   model.mi=model.mi-5;
end

   
  falta calcular h 
            
   
   

   