%************************************************************************
% function model = treina_model(X,S,model,h,ne,ns,N)
% Para mistura de especialista
% Metodo de apresentacao dos padroes em lote
% Mtodo Levenberg Marquardt
%************************************************************************

function model = treina_model(X,Yd,model,h,ne,ns,N)

alfa=1;

% Calcula o likelihood anterior
Likelihood_ant = fobjetiva(X,Yd,model,N,ns);

%Calcula a saida para todos especialistas
for i=1:model.Nm,
   Y(:,:,i)=perceptrons_expert(X,model.models(i).A,model.models(i).B);
end

%Calcula a saida para a rede gating
g=perceptrons_gating(X,model.gating.A,model.gating.B);

%Copia o numero de neuronios
hp=model.hp;
hg=model.hg;

%Calcula o numero de padres da rede especialista
npar_expert= (ne+1)*hp+(hp+1)*ns;

%Calcula o numero de padres da rede gating
npar_gating= (ne+1)*hg+(hg+1)*model.Nm;

%Calcula o numero total de parametros
npar_total=npar_expert*model.Nm+npar_gating;

%Matriz para armazenar as derivadas
for i=1:model.Nm,
     dYdr(:,:,i)=zeros(npar_expert,ns);      
     dJdr(:,i)=zeros(npar_expert,1); 
end

dJdz=zeros(npar_gating,1);
dYdB=zeros(ns*(hp+1),ns);
ddJddx=zeros(npar_total,npar_total);

for k=1:N,
   
   %----------------------------------------------------------------------------
   % Calcula a saida para cada especialista
	%----------------------------------------------------------------------------
   for i=1:model.Nm,     
     %Copia a matriz de pesos para as variaves w1 w2
      A=model.models(i).A;
      B=model.models(i).B;
   	%Calcula a saida para camada escondida   
      x=[X(k,:)';1];
   	y1=tanh(A*x);
      u=[y1;1];
      %Calcula a saida da  rede
      y2=B*u;
      %Calcula a o erro e as derivadas
      yd=Yd(k,:)';
      erro=yd-y2;
      %Calcula deriva dJdY
      dJdY=h(k,i)*erro;
      
      for j=1:hp,
         for l=1:ns,
            dYdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-u(j,1)*u(j,1))*x;
         end
      end
      
      for l=1:ns,
         dYdB(1+(hp+1)*(l-1):(hp+1)*l,l)=u;
      end
      
      dYdr(:,:,i)=[dYdA;dYdB];
      
      
      dJdr(:,i)=dJdr(:,i)+dYdr(:,:,i)*dJdY; 
   end
   
   %--------------------------------------------------------------------------------
   %Calcula a derivada segunda entre os especialista entre si
   %--------------------------------------------------------------------------------
   for i=1:model.Nm,
      for j=1:model.Nm,
         if i==j
            erro=Yd(k,:)'-Y(k,:,i)';
            ddJddY=-h(k,i)*eye(ns,ns)+erro*(h(k,i)-h(k,i)^2)*erro';
			   ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(i-1):npar_expert*i)=...
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(i-1):npar_expert*i)+...
            dYdr(:,:,i)*ddJddY*dYdr(:,:,i)';
      	else
            erro1=Yd(k,:)'-Y(k,:,i)';
            erro2=Yd(k,:)'-Y(k,:,j)';
            ddJdYdY=erro*h(k,i)*h(k,j)*erro';
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(j-1):npar_expert*j)=...
            ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*(j-1):npar_expert*j)+...
            dYdr(:,:,i)*ddJdYdY*dYdr(:,:,j)';
         end
      end
   end
   
   %-------------------------------------------------------------------------------------
   % Calcula a saida da rede gating
   %-------------------------------------------------------------------------------------
   %Copia a matriz de pesos para as variaves w1 w2
   A=model.gating.A;
   B=model.gating.B;
 	%Calcula a saida para camada escondida   
   x=[X(k,:)';1];
  	y1=tanh(A*x);
   u=[y1;1];
   %Calcula a saida da  rede
   y2=B*u;
   %Normaliza
   g=exp(y2)/sum(exp(y2));
   %Calcula a o erro e as derivadas
   yd=h(k,:)';
   erro=yd-g;
   %Calcula deriva dJdY
   dJds=erro;
   
   for j=1:hg,
      for l=1:model.Nm,
         dYdA(1+(ne+1)*(j-1):(ne+1)*j,l)=B(l,j)*(1-u(j,1)*u(j,1))*x;
      end
   end
      
   for l=1:model.Nm,
      dYdB(1+(hg+1)*(l-1):(hg+1)*l,l)=u;
   end
      
   dsdz=[dYdA;dYdB];
   dJdz=dJdz+dsdz*dJds; 
   
	%--------------------------------------------------------------------------------
   %Calcula a derivada segunda para a rede gating
   %--------------------------------------------------------------------------------
   for i=1:model.Nm,
      for j=1:model.Nm,
         if i==j
            ddJdds(i,j) =(h(k,i)-h(k,i)*h(k,i))-(g(i)-g(i)*g(i));
         else
	         ddJdds(i,j) =-h(k,i)*h(k,j)+g(i)*g(j);
         end
      end
   end
   
   ddJddx(1+npar_expert*model.Nm:end,1+npar_expert*model.Nm:end)=...
   ddJddx(1+npar_expert*model.Nm:end,1+npar_expert*model.Nm:end)+...
   dsdz*ddJdds*dsdz';

	%--------------------------------------------------------------------------------
   %Calcula a derivada segunda entre a rede gating e especialista
   %--------------------------------------------------------------------------------
   for i=1:model.Nm,
      erro=Yd(k,:)'-Y(k,:,i)';
      for j=1:model.Nm,
         if i==j
            ddJdsdY(j,:)=(h(k,j)-h(k,j)*h(k,j))*erro';
         else
            ddJdsdY(j,:)=-h(k,i)*h(k,j)*erro';
         end
      end
      ddJddx(1+npar_expert*model.Nm:end,1+npar_expert*(i-1):npar_expert*i)=...
      ddJddx(1+npar_expert*model.Nm:end,1+npar_expert*(i-1):npar_expert*i)+...
      dsdz*ddJdsdY*dYdr(:,:,i)';
   end

      

	%--------------------------------------------------------------------------------
   %Calcula a derivada segunda entre especialista e rede gating
   %--------------------------------------------------------------------------------
  	for i=1:model.Nm,
      erro=Yd(k,:)'-Y(k,:,i)';
      for j=1:model.Nm,
         if i==j
            ddJdYds(:,j)=(h(k,j)-h(k,j)*h(k,j))*erro;
         else
            ddJdYds(:,j)=-h(k,i)*h(k,j)*erro;
         end
      end
      ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*model.Nm:end)=...
      ddJddx(1+npar_expert*(i-1):npar_expert*i,1+npar_expert*model.Nm:end)+...
      dYdr(:,:,i)*ddJdYds*dsdz';
  	end
	
   
end

%--------------------------------------------------------------------------------
%convert as derivadas em vetor
%--------------------------------------------------------------------------------
vetdJdx=[];
% Rede especialista
for i=1:model.Nm,
   vetdJdx=[vetdJdx;dJdr(:,i)];
end

% Rede Gating
vetdJdx=[vetdJdx;dJdz];

%--------------------------------------------------------------------------------
%convert os pesos em vetor
%--------------------------------------------------------------------------------
vetx=convert_vetor(model,ns,ne)

d=inv(ddJddx-model.mi*eye(npar_total))*vetdJdx;

vet_peso=vetx-d;

%--------------------------------------------------------------------------------
%convert os pesos em matriz
%--------------------------------------------------------------------------------

model=convert_matriz(model,vet_peso,ne,ns);

%--------------------------------------------------------------------------------
%verifica o erro
%--------------------------------------------------------------------------------

Likelihood_atual = fobjetiva(X,Yd,model,N,ns);

while Likelihood_atual<Likelihood_ant
   alfa=alfa/2
	vet_peso=vetx-alfa*d;
   model=convert_matriz(model,vet_peso,ne,ns);
   Likelihood_atual = fobjetiva(X,Yd,model,N,ns);
end


Likelihood_ant

Likelihood_atual

alfa

   

  