function [hme] = hmeRead (fname)
% [hme] = hmeRead (fname)
%
% Write a HME model to a file.
%
% INPUTS
%	fname	File name (string).
%
% OUTPUTS
%	hme	HME model.
%
% See also hme.
%
% David Martin <dmartin@eecs.berkeley.edu>
% May 2002

[fid,msg] = fopen(fname,'r');
if fid == -1, error(msg); end
hme = hmeReadTree(fid);
fclose(fid);

function [hme] = hmeReadTree(fid)
  [hme.leaf,count] = fread(fid,1,'int32');
  [sz,count] = fread(fid,[1 2],'int32');
  [hme.param,count] = fread(fid,sz,'double');
  if ~hme.leaf,
    [n,count] = fread(fid,1,'int32');
    hme.children = cell(1,n);
    for i = 1:n,
      hme.children{i} = hmeReadTree(fid);
    end
  end
