function [hme] = hmeCreate(levels,b,d,k)
% [hme] = hmeCreate(levels,b,d,k)
%
% Create balanced-tree HME model.
% The model parameters are initialized to zero.
%
% INPUTS
%	levels	number of levels (>=0) in hierarchy
%	b	branching factor
%	d	dimensionality of data
%	k	number of classes
%
% OUTPUTS
%	hme		HME model
%
% See also hmeCreateExpert, hmeCreateMixture, hmeInitRand,
% hmeInitLR. 
%
% David Martin <dmartin@eecs.berkeley.edu> 
% Charless Fowlkes <fowlkes@eecs.berkeley.edu>
% May 7, 2002

if levels == 0,
  hme = hmeCreateExpert(d,k);
else
  children = cell(1,b);
  for i = 1:b,
    children{i} = hmeCreate(levels-1,b,d,k);
  end
  hme = hmeCreateMixture(children,d);
end

