function [hme] = hmeCreateMixture(children,d,param)
% [hme] = hmeCreateMixture(children,b,d,param)
%
% Create HME model consisting of a mixture of a set of HME models.
% The model parameters are initialized to zero if not provided.
%
% INPUTS
% 	children 	1xb cell array of HME models
%	d		dimensionality of data
%	[param]		dxb matrix of mixing model coefficients
%
% OUTPUTS
%	hme		HME model
%
% See also hmeCreateExpert, hmeInitRand, hmeInitLR. 

% David Martin <dmartin@eecs.berkeley.edu> 
% Charless Fowlkes <fowlkes@eecs.berkeley.edu>
% May 7, 2002

error(nargchk(2,3,nargin));

b = length(children);

hme.leaf = 0;
hme.param = zeros(d,b);
hme.children = children;

if nargin == 3,
  hme.param = param;
end

