function [hme] = hmeCreateExpert(d,k,param)
% [hme] = hmeCreateExpert(d,k,param)
%
% Create HME model consisting of a single expert.
% The model parameters are initialized to zero if not provided.
%
% INPUTS
%	d	dimensionality of data
%	k	number of classes
%	[param]	dxk matrix of model coefficients
%
% OUTPUTS
%	hme	HME model
%
% See also hmeCreateMixture, hmeInitRand, hmeInitLR. 

% David Martin <dmartin@eecs.berkeley.edu> 
% Charless Fowlkes <fowlkes@eecs.berkeley.edu>
% May 7, 2002

error(nargchk(2,3,nargin));

hme.leaf = 1;

disp(sprintf('Escolha o modelo do especialista'))
disp(sprintf('[1]    - Especialista lineares treinados com metodo gradiente'))
disp(sprintf('[2]    - Especialista lineares treinados com pseudo inversao'))
disp(sprintf('[3]    - Redes Feedforward treinados com metodo gradiente'))
disp(sprintf('[4]    - Redes Feedforward treinados com metodo gradiente Conjugado'))
disp(sprintf('[5]    - Redes Feedforward treinados com metodo levenberg'))
disp(sprintf('[6]    - Support Vector Machine'))
disp(sprintf('[7]    - Least Square Support Vector Machine'))
type_expert=input('Escolha a opcao desejada  ');

hme.sig=1; % Assumindo funcao gaussiana unitaria

if type_expert==1
    %hme.param = zeros(d+1,k);
    hme.param = rands(d+1,k);
    hme.type='linear';
    hme.optimizer='gradiente';
elseif type_expert==2
    %hme.param = zeros(d+1,k);
    hme.param = rands(d+1,k);
    hme.type='linear';
    hme.optimizer='pseudo_inversao';    
elseif type_expert==3
    h=input('\nDigite o numero de neuronios para cada Rede especialista  ');
    hme.param.A = rands(h,d+1); % Ja tem bias
    hme.param.B = rands(k,h+1);
    hme.type='mlp';
    hme.h=h;
    hme.optimizer='gradiente';
elseif type_expert==4
    h=input('\nDigite o numero de neuronios para cada Rede especialista  ');
    hme.param.A = rands(h,d+1); % Ja tem bias
    hme.param.B = rands(k,h+1);
    hme.type='mlp';
    hme.h=h;
    hme.optimizer='conjugado';
elseif type_expert==5
    h=input('\nDigite o numero de neuronios para cada Rede especialista  ');
    hme.param.A = rands(h,d+1); % Ja tem bias
    hme.param.B = rands(k,h+1);
    hme.type='mlp';
    hme.h=h;
    hme.optimizer='levenberg';
elseif type_expert==5
    hme.param.alpha=[];
    hme.param.bias=0;
    hme.type='svm';
elseif type_expert==6
    hme.param.alpha=[];
    hme.param.bias=0;
    hme.type='lssvm';
end


if nargin == 3,
    hme.param = param;
end

