function [hme] = hmeCreateMixture(children,d,param)
% [hme] = hmeCreateMixture(children,b,d,param)
%
% Create HME model consisting of a mixture of a set of HME models.
% The model parameters are initialized to zero if not provided.
%
% INPUTS
% 	children 	1xb cell array of HME models
%	d		dimensionality of data
%	[param]		dxb matrix of mixing model coefficients
%
% OUTPUTS
%	hme		HME model
%
% See also hmeCreateExpert, hmeInitRand, hmeInitLR. 

% David Martin <dmartin@eecs.berkeley.edu> 
% Charless Fowlkes <fowlkes@eecs.berkeley.edu>
% May 7, 2002

error(nargchk(2,3,nargin));

b = length(children);

hme.leaf = 0;

disp(sprintf('Modelos de Rede Gating'))
disp(sprintf('[1] - Gating lineares treinados com metodo gradiente'))
disp(sprintf('[2] - Gating lineares treinados com pseudo_inversao'))
disp(sprintf('[3] - Redes Feedforward treinados com metodo gradiente'))
disp(sprintf('[4] - Redes Feedforward treinados com metodo conjugado'))
disp(sprintf('[5] - Redes Feedforward treinados com metodo levenberg'))
disp(sprintf('[6] - Kernels Normalizados'))
type_gating=input('Escolha a opcao desejada  ');

if type_gating==1
    hme.param = zeros(d+1,b);
    %hme.param = 1e-3*rand(d+1,b);
    hme.type='linear';
    hme.optimizer='gradiente';

elseif type_gating==2
    %hme.param = zeros(d+1,b);
    hme.param = 1e-3*rand(d+1,b);
    hme.type='linear';
    hme.optimizer='pseudo_inversao';
    
elseif type_gating==3
    h=input('\nDigite o numero de neuronios para cada Rede especialista  ');
    hme.param.A = rands(h,d+1);
    hme.param.B = rands(b,h+1);
    hme.type='mlp';
    hme.h=h;
    hme.optimizer='gradiente';
    
elseif type_gating==4
    h=input('\nDigite o numero de neuronios para cada Rede especialista  ');
    hme.param.A = rands(h,d+1);
    hme.param.B = rands(b,h+1);
    hme.type='mlp';
    hme.h=h;
    hme.optimizer='conjugado';

elseif type_gating==4
    h=input('\nDigite o numero de neuronios para cada Rede especialista  ');
    hme.param.A = rands(h,d+1);
    hme.param.B = rands(b,h+1);
    hme.type='mlp';
    hme.h=h;
    hme.optimizer='levenberg';
    
elseif type_gating==5
    hme.param.m=[];
    hme.param.sigma=0;
    hme.type='localized';
end

hme.children = children;

if nargin == 3,
  hme.param = param;
end

