function [post,lik,lli] = hmeEval(hme,k,x,y)
% [] = hmeEval(hme,k,x,y)
%
% Evaluate the HME model given samples x and class labels y.
%
% INPUTS
%	hme	HME model
%	k	number of classes
%	x	dxn matrix of samples
%	[y]	kxn vector of class memberships
%
% OUTPUTS
%	post	kxn matrix of class posteriors
%	lik	1xn vector of sample likelihoods (if y is given)
%	lli	log likelihood (if y is given)
%
% Calls logistK_eval.
%
% David Martin <dmartin@eecs.berkeley.edu> 
% Charless Fowlkes <fowlkes@eecs.berkeley.edu>
% May 7, 2002

error(nargchk(3,4,nargin));
if nargin > 3,
 % error(nargoutchk(1,3,nargout));
else
 %error(nargoutchk(1,1,nargout));
end

[d,n] = size(x);

if hme.leaf,

  % evaluate expert
  if nargin > 3,
      switch lower (hme.type)
          case 'linear'
              [post,lik,lli] = linearK_eval(hme.param,hme.sig,x,y);
          case 'mlp'
              [post,lik,lli] = mlpK_eval(hme.param,hme.sig,x,y);
          case 'svm'
              disp('nao implementado')
      end
  else
     switch lower(hme.type)
         case 'linear'
             post = linearK_eval(hme.param,hme.sig,x);
         case 'mlp'
             post = mlpK_eval(hme.param,hme.sig,x);
         case 'svm'
             disp('nao implementado')
     end    
  end

else

    % evaluate gating function
    switch lower(hme.type)
        case 'linear'
            gate = linearKg_eval(hme.param,x);
        case 'mlp'
            gate = mlpKg_eval(hme.param,x);
        case 'Gaussiano'
            disp('Nao implementado')
    end

  % combine post,lik from children using gating function
  post = zeros(k,n);
  lik = zeros(1,n);
  for i = 1:length(hme.children),
    if nargin > 3,
      [post_i,lik_i] = hmeEval(hme.children{i},k,x,y);
      lik = lik + lik_i .* gate(i,:);
    else
      post_i = hmeEval(hme.children{i},k,x);
    end
    post = post + post_i .* repmat(gate(i,:),k,1);
  end

  % log-likelihood
  if nargin > 3,
    lli = sum(log(lik+eps));
  end
  
end

